@@ -2,7 +2,7 @@ voom <- function(counts,design=NULL,lib.size=NULL,normalize.method="none",span=0
 #	Linear modelling of count data with mean-variance modelling at the observational level.
 #	Creates an EList object for entry to lmFit() etc in the limma pipeline.
 #	Gordon Smyth and Charity Law
-#	Created 22 June 2011.  Last modified 9 Mar 2018.
+#	Created 22 June 2011.  Last modified 19 April 2019.
 {
 	out <- list()
 
@@ -43,6 +43,22 @@ voom <- function(counts,design=NULL,lib.size=NULL,normalize.method="none",span=0
 	fit <- lmFit(y,design,...)
 	if(is.null(fit$Amean)) fit$Amean <- rowMeans(y,na.rm=TRUE)
 
+#	If no replication found, set all weight to 1
+	NWithReps <- sum(fit$df.residual > 0L)
+	if(NWithReps < 2L) {
+		if(NWithReps == 0L) warning("The experimental design has no replication. Setting weights to 1.")
+		if(NWithReps == 1L) warning("Only one gene with any replication. Setting weights to 1.")
+		out$E <- y
+		out$weights <- y
+		out$weights[] <- 1
+		out$design <- design
+		if(is.null(out$targets))
+			out$targets <- data.frame(lib.size=lib.size)
+		else
+			out$targets$lib.size <- lib.size
+		return(new("EList",out))
+	}
+
 #	Fit lowess trend to sqrt-standard-deviations by log-count-size
 	sx <- fit$Amean+mean(log2(lib.size+1))-log2(1e6)
 	sy <- sqrt(fit$sigma)
@@ -66,7 +82,7 @@ voom <- function(counts,design=NULL,lib.size=NULL,normalize.method="none",span=0
 #	var0 <- var(log2(0.5*1e6/(lib.size+0.5)))^0.25
 #	var0 <- max(var0,1e-6)
 #	l$y <- c(var0, l$y)
-	f <- approxfun(l, rule=2)
+	f <- approxfun(l, rule=2, ties=list("ordered",mean))
 
 #	Find individual quarter-root fitted counts
 	if(fit$rank < ncol(design)) {
